/*
 * drivers/pci/chpset.c       Version 0.8  1998/2/3
 *
 * Linux PCI subsystem, initialization for Hoontech PCI-Device
 *
 * Copyright (C) 1998 Hoontech co.,Ltd
 * Written By JooKwang-Kim( k2and5@www.hoontech.com )
 */
#include <linux/config.h>
#include <linux/ptrace.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/bios32.h>
#include <linux/pci.h>
#include <linux/string.h>

#include <asm/page.h>

#include "pciio.h"

static DWORD fChipSet=0;
static short gDev=0;
//--------------------------------------------------------------------------
// SIS
//--------------------------------------------------------------------------
DWORD SIS_ReadIRQ( unsigned char bus,  unsigned char dev,
				   unsigned char func, unsigned char reg )
{
	DWORD ret;
	short lsb;

	// hi 3 byte #43-40
	ret = PCI_ReadReg( bus, dev, func, reg );
	// low 1 byte #47-44
	lsb = PCI_ReadReg( bus, dev, func, reg+4 ) & 0xFF;

	// rebuild as 44-43-42-41
	ret >>= 8;
	lsb &= 0xFF;
	ret |= ((long)lsb<<24);

	return ret;
}

void SIS_WriteIRQ( unsigned char bus,  unsigned char dev,
				   unsigned char func, unsigned char reg, DWORD data )
{
	DWORD ret;
	short lsb;

	ret = PCI_ReadReg( bus, dev, func, reg );

	ret &= 0x000000FFL;
	ret |= (data<<8);
	PCI_WriteReg( bus, dev, func, reg, ret );

	ret = PCI_ReadReg( bus, dev, func, ret+4 );
	lsb = (short)((data>>24)&0xFF);
	ret = (ret & ~0x0000FFFFL) | lsb;
	PCI_WriteReg( bus, dev, func, reg+4, ret );
}

//--------------------------------------------------------------------------
// OPTI
//--------------------------------------------------------------------------
void OPTI_TriggerToLevel( unsigned char bus,  unsigned char dev,
						  unsigned char func, short irq )
{
	short t_table[11]={0,5,9,10,11,12,14,15,7,4,3};
	DWORD ret;
	short i;

	for(i=0; i<11; i++) { if (t_table[i]==irq) break; }

	if(i==0 || i==11) return;

	if(i<8)
	{
		ret = PCI_ReadReg( bus, dev, func, 0x40 );
		ret |= (1L<<(i+16));			//0x42
		PCI_WriteReg( bus, dev, func, 0x40, ret );
	}
	else
	{
		i-=8;
		ret = PCI_ReadReg( bus, dev, func, 0x50 );
		ret |= (1L<<(i+8));				//0x51
		PCI_WriteReg( bus, dev, func, 0x50, ret );
	}
}


DWORD OPTI_ReadIRQ( unsigned char bus,  unsigned char dev,
					unsigned char func, unsigned char reg )
{
short irq40[] = {0, 5, 9, 10, 11, 12, 14, 15 };
short irq50[] = {0, 3, 4, 7};

	DWORD r40, r50, ret;
	short i;

	r40 = PCI_ReadReg( bus, dev, func, reg );
	r50 = PCI_ReadReg( bus, dev, func, reg+0x10 );

	ret=0;
	for(i=0; i<4; i++)
	{
		short irq;

		irq = ((r40>>(i*3))&7);
		if(!irq)
		{
			irq = ((r50>>(i*2))&3);
			irq = irq50[ irq ];
		}
		else
			irq = irq40[ irq ];

		if( ! irq ) irq=0x80;

		ret |= (((DWORD)irq)<<(8*i));
	}
	return ret;
}

void OPTI_WriteIRQ( unsigned char bus,  unsigned char dev,
					unsigned char func, unsigned char reg, DWORD data )
{
short irq40[] = {0, 5, 9, 10, 11, 12, 14, 15 };
short irq50[] = {0, 3, 4, 7};

	DWORD r40, r50;
	short i, irq, p, m;

	r40=r50=0L;

	for(i=0; i<4; i++)
	{
		irq = (data>>(8*i))&0xFF;

		if(irq==0x80) irq=0;

		for(p=0; p<4; p++) { if( irq == irq50[p] ) break; }
		if( 0<p && p<4 )
		{
			m=0;
			goto shift_opti;
		}

		for(m=0; m<8; m++) { if( irq == irq40[m] ) break; }
		if( 0<m && m<8 )
		{
			p=0;
//			OPTI_TriggerToLevel( bus, dev, func, irq40[m]);
			goto shift_opti;
		}

		m=p=0;

shift_opti :

		r40 |= (m<<(i*3));
			if(m) { r40 |= (1L<<(m+16));}

		r50 |= (p<<(i*2));
			if(p) { r50 |= (1L<<(11-p));}

	}

	{
		DWORD ret;

		// 2 byte #41-40
		ret = PCI_ReadReg( bus, dev, func, reg );
		ret &= ~0x00FE0FFFL;
		ret |= (r40&0x00FE0FFFL);
		PCI_WriteReg( bus, dev, func, reg, ret );

		// low 1 byte #50
		ret = PCI_ReadReg( bus, dev, func, reg+0x10 );
		ret &= ~0x000007FFL;
		ret |= (r50&0x000007FFL);
		PCI_WriteReg( bus, dev, func, reg, ret );
	}

	r40 = PCI_ReadReg( bus, dev, func, reg );
	r50 = PCI_ReadReg( bus, dev, func, reg+0x10 );
}

//--------------------------------------------------------------------------
// Router
//--------------------------------------------------------------------------

DWORD PCI_ReadRouter( unsigned char bus, unsigned char dev,
					  unsigned char func)
{
	DWORD ret;

	switch( fChipSet )
	{
		case INTEL_7110 :
		case INTEL_7000 : ret=PCI_ReadReg( bus, dev, func, ROUTREG ); break;
		case INTEL_122E : ret=PCI_ReadReg( bus, dev, func, ROUTREG ); break;
		case SIS_0008   : ret=SIS_ReadIRQ( bus, dev, func, 0x40 );    break;
		case OPTI_C558  : ret=OPTI_ReadIRQ( bus, dev, func, 0x40 );   break;
	}

	return ret;
}

DWORD PCI_WriteRouter( unsigned char bus,  unsigned char dev,
					   unsigned char func, DWORD data)
{
	DWORD ret=0L;

	switch( fChipSet )
	{
		case INTEL_7110 :
		case INTEL_7000 : PCI_WriteReg( bus, dev, func, ROUTREG, data ); break;
		case INTEL_122E : PCI_WriteReg( bus, dev, func, ROUTREG, data ); break;
		case SIS_0008   : SIS_WriteIRQ( bus, dev, func, 0x40, data ); break;
		case OPTI_C558  : OPTI_WriteIRQ( bus, dev, func, 0x40, data ); break;
	}

	return ret;
}

char *PCIsetGetChipString( void )
{
	if(fChipSet == INTEL_7110) return "PIIX-4 82371AB";
	if(fChipSet == INTEL_7000) return "PIIX-3 82371SB";
	if(fChipSet == INTEL_122E) return "PIIX-1 82371FB";
	if(fChipSet == SIS_0008)   return "PCIset SIS55x1";
	if(fChipSet == OPTI_C558)  return "OPTI   82C558M";

	return "PCIset UNKNOWN ";
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------

short PCI_BuildIntrMap( void )
{
	DWORD r, i, j;
	short nIRQ;
	char b;

	nIRQ=0;

	for ( i=0; i<0x20; i++ )
	{
		r = PCI_ReadReg( 0, i, 0, 0);

		if( r != (DWORD)-1 )
		{
			for(j=0; j<8; j++)
			{
				r = PCI_ReadReg( 0, i, j, 0x3c);

				if(((r&0xFF) != (DWORD)0xFFL)&&r)
				{
					b = (char)(r&0xF);
					nIRQ |= (1<<b);
				}
			}
		}
	}

	return nIRQ;
}

void PCIsetClearUnUsed( short nDev )
{
	DWORD r, mask=0xFF;
	short pin, rMap, nIrq;

	rMap = PCI_BuildIntrMap();
	r 	 = PCI_ReadRouter( 0, nDev, 0 );

	for( pin=0; pin<4*8; pin+=8)
	{
		// Check used ?
		if( ((r>>pin)&0x80)!=0x80 )
		{
			nIrq = (r>>pin)&0xF;

			if( rMap & (1<<nIrq) )
			;
			else
			{
				r &= ~(mask<<pin);
				r |= ((long)0x80<<pin);
			}
		}
	}

	PCI_WriteRouter( 0, nDev, 0, r );
	r = PCI_ReadRouter( 0, nDev, 0 );
}

void PCIsetSetATXDDMA( DWORD val )
{
	DWORD r,i;

	for ( i=0; i<0x20; i++ )
	{
		r = PCI_ReadReg( 0, i, 0, 0);

		if ( r==INTEL_7110 )
		{
			PCI_WriteReg( 0, i, 0, 0x90, val );
		}
	}
}

short PCIsetAvail( short nDev )
{
	DWORD r;
	short pin, ret;

	r = PCI_ReadRouter( 0, nDev, 0 );
	ret=0;

	for( pin=0; pin<4*8; pin+=8)
	{
		// Check used ?
		if( ((r>>pin)&0x80)==0x80 )
		{
			ret++;
		}
	}

	return ret;
}

short PCIsetIsDupliIrq( short nDev, short irq )
{
	DWORD r;
	short pin;

	r = PCI_ReadRouter( 0, nDev, 0 );

	for( pin=0; pin<4*8; pin+=8)
	{
		if( ((r>>pin)&0xF)==irq )
		return 1;
	}

	return 0;
}

short PCIsetSetIrq( short nDev, short pin, short irq )
{
	DWORD r, mask=0xFF;
	short ret=0;

	pin *= 8;

	r = PCI_ReadRouter( 0, nDev, 0 );

	// Check Empty ?

	if( (r>>pin)&0x80 )
	{
		r &= ~(mask<<pin);
		r |= ((long)irq<<pin);
		PCI_WriteRouter( 0, nDev, 0, r );
		ret=1;
	}
	return ret;
}

void PCIsetSetChipset( DWORD dwChipset, short nDev )
{
	switch( dwChipset )
	{
		case INTEL_7110 : gDev=nDev; fChipSet = INTEL_7110; break;
		case INTEL_7000 : gDev=nDev; fChipSet = INTEL_7000; break;
		case INTEL_122E : gDev=nDev; fChipSet = INTEL_122E; break;
		case SIS_0008   : gDev=nDev; fChipSet = SIS_0008;   break;
		case OPTI_C558  : gDev=nDev; fChipSet = OPTI_C558;  break;
	}
}

int PCIsetIsATX( void )
{
	if( fChipSet==INTEL_7110 ) return 1;
	return 0;
}

BOOL IsChipsetFinded( void )
{
	return (fChipSet!=0);
}


short PCIsetSetTestOn( short pin, short irq )
{
	short ret = 0;

	PCIsetClearUnUsed( gDev );

	if( PCIsetAvail( gDev ) && !PCIsetIsDupliIrq( gDev, irq ) )
	{
		ret = PCIsetSetIrq( gDev, pin, irq );
	}
	return ret;
}
