/*
 * drivers/pci/cspci.c       Version 0.8  1998/2/3
 *
 * Linux PCI subsystem, initialization for CS4237
 *
 * Copyright (C) 1998 Hoontech co.,Ltd
 * Written By JooKwang-Kim( k2and5@www.hoontech.com )
 */
#include <linux/config.h>
#include <linux/ptrace.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/bios32.h>
#include <linux/pci.h>
#include <linux/string.h>

#include <asm/page.h>
#include <asm/io.h>


#include "pciio.h"

#define MSWAIT	   100
#define PnpADDR    0x338 //0x279
#define arrsize(x) (sizeof(x)/sizeof(x[0]))

void wait(int time)
{
	int i;
	for (i=0;i<time;i++) IN_P( 0xee );
}

void SendCrystalKey( void )
{
	int i;
	BYTE bCrystalKey[32] =
	{
		0x96,0x35,0x9a,0xcd,0xe6,0xf3,0x79,0xbc,
		0x5e,0xaf,0x57,0x2b,0x15,0x8a,0xc5,0xe2,
		0xf1,0xf8,0x7c,0x3e,0x9f,0x4f,0x27,0x13,
		0x09,0x84,0x42,0xa1,0xd0,0x68,0x34,0x1a
	};

	for (i=0;i<arrsize(bCrystalKey);i++)
	{
		OUT_P( PnpADDR, bCrystalKey[i] ); wait(MSWAIT);
	}
}

//------------------------------------
//	SetFMMode
//
//	0:External
//	1:Internal
//
//------------------------------------
void SendHardConf( int bFmMode, int wPortCTRL )
{
	int i;
	BYTE CSHwCfg[19]=
	{
							0x00,0x03,0xC2,0x00, //0x00 -> for EXTFM enable
		0x0B,0x20,0x04,0x08,0x10,0x80,0x00,0x00,
		0x00,0x48,0x75,0xB9,0xFC,0x10,0x03
	};

	switch(bFmMode)
	{
		case 0 : CSHwCfg[3]=0x80; 	break; // Internal FM
		case 1 : CSHwCfg[3]=0; 		break; // External FM
	}

	for(i=0;i<arrsize(CSHwCfg);i++)
	{
		OUT_P( wPortCTRL+5, CSHwCfg[i]) ; wait(MSWAIT);
	}
}

void SendPnpRes( int wPortCTRL )
{
	int i;

	BYTE bPnpRes[] =
	{

	// CS4237 FFFFFFFF
	0x0E, 0x63, 0x42, 0x37, 0xFF, 0xFF, 0xFF, 0xFF, 0x30,

	// PnP version 1.0, Vender version 0.1
	0x0A, 0x10, 0x01,

	// ANSI ID: CMB4237B
	0x82, 0x09, 0x00, 'C', 'M', 'B', '4', '2', '3', '7', 'B', 0x00,

// LOGICAL DEVICE 0 (Windows Sound System & OPL3 & SBPro)

	0x15, 0x0E, 0x63, 0x00, 0x00, 0x00,				// EISA ID: CSC0000
	0x82, 0x07, 0x00, 'W','S','S','/','S','B', 0x00,// ANSI ID

	0x31, 0x00,                                     // DF Acceptable Choice
	0x2A, 0x02, 0x28,                               // DMA: 1  - WSS & SBPro
	0x2A, 0x09, 0x28,                               // DMA: 0,3  - WSS & SBPro capture
	0x22, 0x20, 0x00,                               // IRQ: 5  Interrupt Select 0
	0x47, 0x01, 0x34, 0x05, 0x34, 0x05, 0x04, 0x04, // 10b WSSbase: 534
	0x47, 0x01, 0x88, 0x03, 0x88, 0x03, 0x08, 0x08, // 10b SYNbase: 388
	0x47, 0x01, 0x20, 0x02, 0x20, 0x02, 0x20, 0x10, // 10b SBbase: 220

	0x31, 0x01,                                     // DF Acceptable Choice
	0x2A, 0x0A, 0x28,                               // DMA: 0,1,3  - WSS & SBPro
	0x2A, 0x0B, 0x28,                               // DMA: 0,1,3  - WSS & SBPro capture
	0x22, 0xA0, 0x9A,                               // IRQ: 5,7,9,11,12,15  Interrupt Select 0
	0x47, 0x01, 0x34, 0x05, 0x08, 0x06, 0xD4, 0x04, // 10b WSSbase: 300-360
	0x47, 0x01, 0x88, 0x03, 0x88, 0x03, 0x08, 0x08, // 10b SYNbase: 100-3FC
	0x47, 0x01, 0x20, 0x02, 0x60, 0x02, 0x20, 0x10, // 10b SBbase: 220-260

	0x31, 0x02,                                     // DF Acceptable Choice
	0x2A, 0x0B, 0x28,                               // DMA: 0,1,3  - WSS & SBPro
	0x22, 0xA0, 0x9A,                               // IRQ: 5,7,9,11,12,15  Interrupt Select 0
	0x47, 0x01, 0x34, 0x05, 0xFC, 0x0F, 0x04, 0x04, // 10b WSSbase: 300-360
	0x47, 0x01, 0x88, 0x03, 0xF8, 0x03, 0x08, 0x08, // 10b SYNbase: 100-3FC
	0x47, 0x01, 0x20, 0x02, 0x00, 0x03, 0x20, 0x10, // 10b SBbase: 220-260

	0x38,                                           // End of DF for Logical Device 0

// LOGICAL DEVICE 1 (Game Port)

	0x15, 0x0E, 0x63, 0x00, 0x01, 0x00,             // EISA ID: CSC0001
	0x82, 0x05, 0x00, 'G','A','M','E', 0x00,        // ANSI ID

	0x31, 0x00,                                     // DF Acceptable Choice
	0x47, 0x01, 0x00, 0x02, 0x00, 0x02, 0x08, 0x08, // 10b GAMEbase: 200

	0x31, 0x01,                                     // DF Acceptable Choice
	0x47, 0x01, 0x08, 0x02, 0x08, 0x02, 0x08, 0x08, // 10b GAMEbase: 200

	0x38,                                           // End of DF for Logical Device 1


// LOGICAL DEVICE 2 (Control)

	0x15, 0x0E, 0x63, 0x00, 0x10, 0x00,             // EISA ID: CSC0010
	0x82, 0x05, 0x00, 'C','T','R','L', 0x00,        // ANSI ID

	0x47, 0x01, 0x20, 0x01, 0xF8, 0x03, 0x08, 0x08, // 10b CTRLbase: 100-3F8


// LOGICAL DEVICE 3 (MPU-401)

	0x15, 0x0E, 0x63, 0x00, 0x03, 0x00,             // EISA ID: CSC0003
	0x82, 0x04, 0x00, 'M','P','U', 0x00,            // ANSI ID

	0x31, 0x00,                                     // DF Acceptable Choice
	0x22, 0x00, 0x02,                               // IRQ: 9  Interrupt Select 0
	0x47, 0x01, 0x30, 0x03, 0x30, 0x03, 0x08, 0x02, // 10b MPUbase: 100-3F8

	0x31, 0x01,                                     // DF Acceptable Choice
	0x22, 0x00, 0x9A,                               // IRQ: 9  Interrupt Select 0
	0x47, 0x01, 0x30, 0x03, 0x60, 0x03, 0x08, 0x02, // 10b MPUbase: 100-3F8

	0x31, 0x02,                                     // DF Acceptable Choice
	0x47, 0x01, 0x30, 0x03, 0xE0, 0x03, 0x08, 0x02, // 10b MPUbase: 100-3F8

	0x38,                                           // End of DF for Logical Device 3


	0x79, 0xD8,                                     // End of Resource Data, Checksum

	};

	for(i=0;i<arrsize(bPnpRes);i++)
	{
		OUT_P( wPortCTRL+5, bPnpRes[i]) ; wait(MSWAIT);
	}
}

//------------------------------------
//	HostLoad
//	Please refer CS4237B DataSheet
//  page number - 24
//
//  Crystal Key Method
//------------------------------------
void HostLoad( int bFmMode, int wPortCTRL )
{
	int i;
	BYTE DevCTRL[]=
	{
		// LDN = 2
		0x15,0x02,         //
		0x47,0x01,0x20,    // CTRL Base
		0x33,0x01,         // activate
		0x79
	};


	DevCTRL[3]= (unsigned char)((wPortCTRL>>8)&0xFF);
	DevCTRL[4]= (unsigned char)(wPortCTRL&0xFF);

	// Send Crystal Key
	SendCrystalKey();

	// Configure Logical Dev 2
	for(i=0;i<arrsize(DevCTRL);i++)
	{
		OUT_P( PnpADDR, DevCTRL[i]) ; wait(MSWAIT);
	}

	// Jump
	OUT_P( wPortCTRL+5, 0x57); wait(MSWAIT);

	// Download
	OUT_P( wPortCTRL+5, 0xAA); wait(MSWAIT);

	// Start Download Address
	OUT_P( wPortCTRL+5, 0x00); wait(MSWAIT);
	OUT_P( wPortCTRL+5, 0x40); wait(MSWAIT);

	// Begin Download resource

	// 1. Hardware configure data - CSHwCfg
	SendHardConf( bFmMode, wPortCTRL );


	// 2. PnP Resource data - bPnpRes
	SendPnpRes( wPortCTRL);

	// End Download resource
	OUT_P( wPortCTRL+6, 0x00 ); wait(MSWAIT);
	OUT_P( wPortCTRL+5, 0x5A ); wait(MSWAIT);
}

void SLAM( void )
{
	int i;
	BYTE bPnpSLAM[] =
	{

	// LDN = 0
	0x15,0x00,
	0x47,0x05,0x34,     // WSS Base
	0x48,0x03,0x88,     // SYN Base
	0x42,0x02,0x20,     // SB Base
	0x22,0x05,          // IRQ 05
	0x2a,0x01,          // DMA0
	0x25,0x00,          // DMA1
	0x33,0x01,          // activate

	// LDN = 1
	0x15,0x01,
	0x47,0x02,0x00,     // Game Base
	0x33,0x01,          // activate

	// LDN = 2
	0x15,0x02,          // CTRL Base
	0x47,0x01,0x20,     // CTRL Base
	0x33,0x01,          // activate

	//LDN = 3
	0x15,0x03,
	0x47,0x03,0x30,     // MPU Base
	0x22,0x09,          // MPU IRQ
	0x33,0x01,          // activate

	0x79                // activate crystal device

	};


	// Send Crystal Key
	SendCrystalKey();

	// PnP Resource data - SLAM
	for(i=0;i<arrsize(bPnpSLAM);i++)
	{
		OUT_P( PnpADDR, bPnpSLAM[i]) ; wait(MSWAIT);
	}
}

void CS4237Main( int bFmMode )
{
	HostLoad( bFmMode, 0x120 );
	SLAM();

	// *** patch for joystick ****
	OUT_P( 0x200, 0 );
	OUT_P( 0x200, 0 );
}
