/*
 * drivers/pci/modemset.c       Version 0.8  1998/2/3
 *
 * Linux PCI subsystem, initialization for Hoontech ModemTrack PCI-56k
 *
 * Copyright (C) 1998 Hoontech co.,Ltd
 * Written By JooKwang-Kim( k2and5@www.hoontech.com )
 */
#include <linux/config.h>
#include <linux/ptrace.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/bios32.h>
#include <linux/pci.h>
#include <linux/string.h>

#include <asm/page.h>
#include <asm/io.h>

#include "pciio.h"

//--------------------------------------------------------------------------
// Globals variable
//--------------------------------------------------------------------------
typedef struct { short port, irq; } COM_DESC;
static COM_DESC sComs[8] =
	{
		{0x3F8, 4}, //COMM 1
		{0x2F8, 3}, //COMM 2
		{0x3E8, 4}, //COMM 3
		{0x2E8, 3}, //COMM 4
		{0x3F8, 7}, //COMM 5
		{0x2F8, 7}, //COMM 6
		{0x3E8, 5}, //COMM 7
		{0x2E8, 5}  //COMM 8
	};

//--------------------------------------------------------------------------
// Extern of PICIO.C
//--------------------------------------------------------------------------
static DWORD reg740,
	  reg744=0x4681,	// move sam9407 to top (sam-sound)
	  reg748,
	  reg74C=0x339,
	  reg750=0,
	  reg754=0,
	  reg758=0;

static DWORD regCFG[8][0x40];

//--------------------------------------------------------------------------
// Method for Global Variable
//--------------------------------------------------------------------------
void InitModemCFG( void )
{
	short i,j;

	for(i=0; i<5; i++)
		for(j=0; j<0x40; j++)
			regCFG[i][j] = (DWORD)-1;
}

//--------------------------------------------------------------------------
//	SetDevice
//--------------------------------------------------------------------------
void SetModemDevice( void )
{
	reg740 = 0;
	reg744 = 0x4681;        // move sam9407 to top
	reg748 = 0;
	reg74C = 0;     	    // no pnp address
	reg750=reg754=reg758=0; // no dma

	reg740 |= 0x0018;				// sam 9407, adlib
	reg748 |= 0xC0;                 // irq enable
	regCFG[1][4]=1;					// address enable
	regCFG[1][0x10]=sComs[0].port;  // sam9407  0-3 address
	regCFG[1][0x14]=sComs[0].port+4;// samadlib 4-7 address

	regCFG[1][0x3C]=0xFF;

	reg748 |= ((DWORD)(0)<<4);
}

static void PCIModem_WriteUSER( unsigned char dev )
{
	unsigned char func = Get_MappedFuncNumber( reg744, 7 );

	PCI_WriteOneReg( dev, func, 0x40, reg740);
	PCI_WriteOneReg( dev, func, 0x44, reg744);
	PCI_WriteOneReg( dev, func, 0x48, reg748);
	PCI_WriteOneReg( dev, func, 0x4C, reg74C);
	PCI_WriteOneReg( dev, func, 0x50, reg750);
	PCI_WriteOneReg( dev, func, 0x54, reg754);
	PCI_WriteOneReg( dev, func, 0x58, reg758);
}

short PCI_InitModemHW( unsigned char dev )
{
	short i,j;
	unsigned char func;

	// USER register
	PCIModem_WriteUSER( dev );

	// PCI standard register
	for(i=0; i<5; i++)
	{
		for(j=0; j<0x40; j+=4)
		{
			if( regCFG[i][j] != (DWORD)-1 ) // 0xFFFFFFFF
			{
				func = Get_MappedFuncNumber( reg744,i );
				PCI_WriteOneReg( dev, func, j, regCFG[i][j] );
			}
		}
	}

	return 1;
}
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
void SetPCIModemAddress( unsigned char dev, short add )
{
	unsigned char func = Get_MappedFuncNumber( reg744,1 );

	regCFG[1][0x10]=add;  // sam9407  0-3 address
	regCFG[1][0x14]=add+4;// samadlib 4-7 address

	PCI_WriteOneReg( dev, func, 0x10, regCFG[1][0x10] );
	PCI_WriteOneReg( dev, func, 0x14, regCFG[1][0x14] );
}

short SetPCIModemHardwareInt( unsigned char dev, short FuncNum, short IRQNum, short IntPinNum )
{
	short pin = IntPinNum-0xa;
	switch( FuncNum )
	{
		case 0 : // wss,sb
			reg748 &= ~0x3;
			reg748 |= (pin&0x3);
			break;
		case 1 : // dream
			reg748 &= ~0x30;
			reg748 |= ((pin&0x3)<<4);
			break;
		case 3 : // mpu
			reg748 &= ~0x300;
			reg748 |= ((pin&0x3)<<8);
			break;
	}
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,7 ), 0x48, reg748);
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,FuncNum ), 0x3C, IRQNum );
	return 1;
}


//--------------------------------------------------------------------------
//	setup_modemtrack
//--------------------------------------------------------------------------
BOOL setup_modemtrack( unsigned char dev )
{
	short dq,com;
	short i, pin;

	dq=0;

	InitModemCFG();

	SetModemDevice( );
	PCI_InitModemHW( dev );

	for(com=0; com<8; com++)
	{
		dq = sComs[com].irq;

		SetPCIModemAddress( dev, 0xFF );
		if( IN_P(sComs[com].port) != 0xFF ) continue;

		for(pin=0; pin<4; pin++)
		{
			SetPCIModemHardwareInt( dev, 1, 0xFF, 0xA+pin);
			for(i=0; i<4; i++)
			{
				if( PCIsetSetTestOn( i, dq ) )
				{
					SetPCIModemAddress( dev, sComs[com].port );
					SetPCIModemHardwareInt( dev, 1, dq, 0xA+pin);
					goto autosuss;
				}
			}
		}
	}

	if( com >= 8 )
	{
		printk("MODEM IRQ Fail %d\n", dq );
		return FALSE;
	}

autosuss :
	printk("Modem Track II ");
	printk( ": %s", PCIsetGetChipString());
	printk(" : (-COM%d-) (PORT=0x%X IRQ=%d)\n", com+1, sComs[com].port, sComs[com].irq);

	return TRUE;
}
