/*
 * drivers/pci/pciio.c       Version 0.8  1998/2/3
 *
 * Linux PCI subsystem, initialization for Hoontech PCI-Device
 *
 * Copyright (C) 1998 Hoontech co.,Ltd
 * Written By JooKwang-Kim( k2and5@www.hoontech.com )
 */
#include <linux/config.h>
#include <linux/ptrace.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/bios32.h>
#include <linux/pci.h>
#include <linux/string.h>

#include <asm/page.h>
#include <asm/io.h>

#include "pciio.h"

//--------------------------------------------------------------------------
// Global Variable
//--------------------------------------------------------------------------
#define DEVFN( dev, func ) (((dev&0x1F)<<3) | (func&0x7))

DWORD PCI_ReadReg( unsigned char bus,  unsigned char dev,
				   unsigned char func, unsigned char reg )
{
	DWORD value=0;

	pcibios_read_config_dword ( bus, DEVFN(dev,func),reg, 
				(unsigned int*)&value );
	return value;
}

void PCI_WriteReg( unsigned char bus,  unsigned char dev,
				   unsigned char func, unsigned char reg, DWORD data )
{
	pcibios_write_config_dword (bus, DEVFN(dev,func),reg, 
				(unsigned int)data );
}

unsigned char Get_MappedFuncNumber( DWORD reg, unsigned char func )
{
	if( func == 7 ) return func;
	else return (reg >> (func*3)) & 0x0007;
}

DWORD PCI_ReadOneReg( unsigned char dev, unsigned char func, unsigned char reg )
{
	return PCI_ReadReg( 0, dev, func, reg );
}

void PCI_WriteOneReg( unsigned char dev, unsigned char func, unsigned char reg, DWORD val )
{
	// Be carefull for IRQ
	if( reg == 0x3c )
	{
		DWORD r=0;

		r = PCI_ReadReg( 0, dev, func, reg );
		r &= (~0x000000FF);
		r |= val;
		PCI_WriteReg( 0, dev, func, reg, r );
	}
	else
	{
		PCI_WriteReg( 0, dev, func, reg, val );
	}
}

int PCI_BuildIntrField( void )
{
	unsigned char i, j;
	DWORD r;
	int nIRQ;
	char b;

	nIRQ=0;

	for ( i=0; i<0x20; i++ )
	{
		r = PCI_ReadReg( 0, i, 0, 0 );

		// skip myself
		if( (WORD)r==(WORD)PCI_VENDOR_ID_HOONTECH )
			continue;

		if( r != (DWORD)-1 )
		{
			for(j=0; j<8; j++)
			{
				r = PCI_ReadReg( 0, i, j, 0x3c);
				if(((r&0xFF) != (DWORD)0xFFL)&&r)
				{
					b = (char)(r&0xF);
					nIRQ |= (1<<b);
				}
			}
		}
	}
	return nIRQ;
}


//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
short KSOUND_IsFirstTime( unsigned char dev, unsigned char func )
{
	return ( PCI_ReadReg( 0, dev, func, 4 ) == 0xFFFFFFFFL ) ? 1 : 0;
}

short KSOUND_IsModem( unsigned char dev )
{
	short add, ret1, ret2, first;
	DWORD reg1=0,reg2=0;
	
	// verify that there is nothing
	for( add=0x908; ; add++)
		if( IN_P(add)==0xff ) break;

	first = KSOUND_IsFirstTime( dev, 0 );

	if( first )
	{
		PCI_WriteReg( 0, dev, 7, 0x40, 0x18L );		// eanble modem device
		PCI_WriteReg( 0, dev, 0, 4, 1 );				// eanble address access

	}
	else
	{
		reg1 = PCI_ReadReg( 0, dev, 0, 0x10 )-1;
		reg2 = PCI_ReadReg( 0, dev, 0, 0x14 )-1;
	}

	PCI_WriteReg( 0, dev, 0, 0x10, (DWORD)(add) );      // enable tempo add
	PCI_WriteReg( 0, dev, 0, 0x14, (DWORD)(add+4) );	// enable tempo add

	// read tempo port
	ret1 = IN_P( add+6 );	// 0x332(Dream no port) or 0x3fA(modem)
	ret2 = IN_P( add+7 );	// 0x332(Dream no port) or 0x3fA(modem)

	if( first )
		PCI_WriteReg( 0, dev, 7, 0x40, 0 );			// clear modem device
	else
	{
		PCI_WriteReg( 0, dev, 0, 0x10, reg1 );			// restore
		PCI_WriteReg( 0, dev, 0, 0x14, reg2 );			// restore
	}

	return ((ret1==0xFF) && (ret2==0xFF) ? 0 : 1);
}

BOOL setup_modemtrack( unsigned char dev );
BOOL setup_soundtrack( unsigned char dev );

void setup_HoonTechPCIDevice( void )
{
	unsigned char dev;
	DWORD ret;

	for(dev=0; dev<0x20; dev++)
	{
		ret = PCI_ReadReg( 0, dev, 0, 0 );

		if( ret != 0xffffffff ) PCIsetSetChipset( ret, dev );

		if( (WORD)ret==(WORD)PCI_VENDOR_ID_HOONTECH )
		{
			if( !IsChipsetFinded())
			{
				printk("PCIset Not Found\n");
				return;
			}

			// sound
			if( !KSOUND_IsModem( dev ) )
				setup_soundtrack( dev );

			// modem
			if( KSOUND_IsModem( dev ) )
				setup_modemtrack( dev );
		}
	}
	return;
}

