/*
 * drivers/pci/soundset.c       Version 0.8  1998/2/3
 *
 * Linux PCI subsystem, initialization for Hoontech SoundTrack PCI-97
 *
 * Copyright (C) 1998 Hoontech co.,Ltd
 * Written By JooKwang-Kim( k2and5@www.hoontech.com )
 */
#include <linux/config.h>
#include <linux/ptrace.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/bios32.h>
#include <linux/pci.h>
#include <linux/string.h>

#include <asm/page.h>
#include <asm/io.h>


#include "pciio.h"

// interrupt pin define

#define PIN_A	0
#define PIN_B	1
#define PIN_C   1
#define DDMA_CHN 	0x000A
#define DDMA_BASE  	0x0500

//Default------------------------------------------------
short nDefDRM  = 0x330;     	// D=  Dream address
short nDefDIRQ = 0x7;			// DI= Dream 	irq
short nDefSB   = 0x220;     	// S=  SB 	 address
short nDefWSS  = 0x534;     	// W=  WSS 	 address
short nDefOPL  = 0x388;     	// O=  OPL   address
short nDefSIRQ = 0x5;       	// SI= SB   	irq
short nDefPDMA = 0x1;       	// PD= Play 	dma
short nDefCDMA = 0x0;       	// CD= Capture 	dma
short nDefMPU  = 0x320;     	// M=  MPU   address
short nDefGAME = 0x200;     	// G=  GAME  address

short nDefCTRL = 0x120;     	// CTRL=  CTRL address for crystal chipset

short nDefSPIN = 0x0;     		// SPIN=SB 		Int Pin
short nDefDPIN = 0x1;     		// DPIN=DREAM 	Int Pin
short nDefMPIN = 0x1;     		// MPIN=MPU 	Int Pin


//--------------------------------------------------------------------------
//
// Extern of PICIO.C
//
//--------------------------------------------------------------------------
static DWORD reg740=0,
	  reg744=0x4688,  // sound-sam
	  reg748=0,
	  reg74C=0x339,
	  reg750=0,
	  reg754=0,
	  reg758=0;


static DWORD regCFG[5][0x40];

//--------------------------------------------------------------------------
//	Mixer_Out
//--------------------------------------------------------------------------
void Mixer_Out(unsigned reg, unsigned val)
{
	int v = ((val<<4)|(val&0xf));
	OUT_P(nDefSB+0x4, (char)reg);
	OUT_P(nDefSB+0x5, (char)v);
}

void InitSoundCFG( void )
{
	short i,j;

	for(i=0; i<5; i++)
		for(j=0; j<0x40; j++)
			regCFG[i][j] = (DWORD)-1;
}


static void PCISound_WriteUSER( unsigned char dev )
{
	unsigned char func = Get_MappedFuncNumber( reg744, 7 );

	PCI_WriteOneReg( dev, func, 0x40, reg740);
	PCI_WriteOneReg( dev, func, 0x44, reg744);
	PCI_WriteOneReg( dev, func, 0x48, reg748);
	PCI_WriteOneReg( dev, func, 0x4C, reg74C);
	PCI_WriteOneReg( dev, func, 0x50, reg750);
	PCI_WriteOneReg( dev, func, 0x54, reg754);
	PCI_WriteOneReg( dev, func, 0x58, reg758);
}

short PCISound_InitHW( unsigned char dev )
{
	short i,j;
	unsigned char func;

	// USER register
	PCISound_WriteUSER( dev );

	// PCI standard register
	for(i=0; i<5; i++)
	{
		for(j=0; j<0x40; j+=4)
		{
			if( regCFG[i][j] != (DWORD)-1 ) // 0xFFFFFFFF
			{
				func = Get_MappedFuncNumber( reg744, i);
				PCI_WriteOneReg( dev, func, j, regCFG[i][j] );
			}
		}
	}
	return 1;
}

void SetPCI16intrpt( unsigned char dev, int nSirq, int nSpin, int nDirq, int nDpin )
{
	// Only enable intrpt for sound and midi
	reg748=0xCC;

	reg748|=nSpin;
	reg748|=(nDpin<<4);

	// Enable K-SOUND intrpt
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,7), 0x48, reg748);

	// Set Sound Intrpt
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,0), 0x3C, nSirq);

	// Set Dream Intrpt
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,1), 0x3C, nDirq);
}

short SetPCISoundHardwareInt( unsigned char dev, short FuncNum, short IRQNum, short IntPinNum )
{
	short pin = IntPinNum-0xa;
	switch( FuncNum )
	{
		case 0 : // wss,sb
			reg748 &= ~0x3;
			reg748 |= (pin&0x3);
			break;
		case 1 : // dream
			reg748 &= ~0x30;
			reg748 |= ((pin&0x3)<<4);
			break;
		case 3 : // mpu
			reg748 &= ~0x300;
			reg748 |= ((pin&0x3)<<8);
			break;
	}
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,7 ), 0x48, reg748);
	PCI_WriteOneReg( dev, Get_MappedFuncNumber( reg744,FuncNum ), 0x3C, IRQNum );
	return 1;
}


//--------------------------------------------------------------------------
//
//	SetDevice
//
//--------------------------------------------------------------------------

void SetSoundDevice( void )
{
	reg740 |= 0x80008; reg748 |= 0xC0; regCFG[1][0x10]=nDefDRM; regCFG[1][4]=1;
	regCFG[1][0x3C]=0xFF;

	reg740 |= 1; reg748 |= 0xC; regCFG[0][0x10]=nDefWSS; regCFG[0][4]=5;
	reg740 |= 2; regCFG[0][0x14]=nDefSB;  regCFG[0][4]=5;
	reg740 |= 4; regCFG[0][0x18]=nDefOPL; regCFG[0][4]=5;

	regCFG[0][0x3C]=0xFF;

	reg740 |= ((DWORD)nDefPDMA<<14);
	reg740 |= ((DWORD)nDefCDMA<<12);

	reg740 |= 0x00200020L; regCFG[2][0x10]=nDefGAME; regCFG[2][4]=1;

	reg740 |= 0x80; regCFG[4][0x10]=nDefCTRL; regCFG[4][4]=1;

	reg740 |= 0x40; regCFG[3][0x10]=nDefMPU;  regCFG[3][4]=1;

	reg748 |= (PIN_A);
	reg748 |= ((DWORD)nDefDPIN<<4);
	reg748 |= (PIN_B<<4);

	reg748 |= ((DWORD)nDefMPIN<<8);
}


void LoaderCSC4237( void )
{
	CS4237Main( 0 );
	Mixer_Out(0x22,0xf);	//master of SB
}

int IRQ_GetNext( int irq )
{
	int ai[]={5,7,9,10,11,12,13};
	int i;

	for(i=0; i<(sizeof(ai)/sizeof(ai[0])); i++)
		if(ai[i]==irq) { i++; break; }

	if( i==(sizeof(ai)/sizeof(ai[0])) )
		return ai[0];
	else
		return ai[i];
}

int IRQ_GetBest( int bField, int irq )
{
	while ( bField & (1<<irq) )
	{
		irq=IRQ_GetNext( irq );
	}
    return irq;
}

BOOL setup_soundtrack( unsigned char dev )
{
	int sq=0,dq=0;
	int bField;
	int i, pin;

	InitSoundCFG();

	SetPCI16intrpt( dev, nDefSIRQ, nDefSPIN, nDefDIRQ, nDefDPIN);

	bField = PCI_BuildIntrField();

	SetSoundDevice( );

	if ( PCIsetIsATX() )
	{
		// program 82371ab
		PCIsetSetATXDDMA( ((DWORD)DDMA_BASE<<16)+DDMA_CHN  );

		// program ksound

		reg750 = 0;				// clear legacy dma
		reg754 = DDMA_BASE+0x1;	// capture ddma
		reg758 = DDMA_BASE+0x11;	// play ddma

		// program ksound to enable mpeg ddma master

		regCFG[4][4]=5;
	}
	else
	{
		// program ksound
		reg750 = 0xB;	// capture and play legacy dma
		reg754 = 0;		// clear capture ddma
		reg758 = 0;		// clear play ddma

		// program ksound to disable mpeg ddma master
		regCFG[4][4]=1;
	}

	PCISound_InitHW( dev );

	if ( PCIsetIsATX() )
	{
		// initialize ddma port
		OUT_P( DDMA_BASE+0x3, 0 );
		OUT_P( DDMA_BASE+0x6, 0 );
		OUT_P( DDMA_BASE+0x8, 0 );	// channel 0 capture

		OUT_P( DDMA_BASE+0x13, 0 );
		OUT_P( DDMA_BASE+0x16, 0 );
		OUT_P( DDMA_BASE+0x18, 0 );// channel 1 play
	}


	//===============================
	LoaderCSC4237();

//--------------------------------------------------------------------------
// ChipSet Irq test
//--------------------------------------------------------------------------
	bField = PCI_BuildIntrField();

	sq = IRQ_GetBest( bField, nDefSIRQ );

	for(pin=0; pin<4; pin++)
	{
		SetPCISoundHardwareInt( dev, 0, 0xFF, 0xA+pin);//atoi(sCmdSPIN));

		for(i=0; i<4; i++)
		{
			if( PCIsetSetTestOn( i, sq ) )
			{
			  SetPCISoundHardwareInt( dev, 0, sq, 0xA+pin);//atoi(sCmdSPIN));
			  goto sqsuss;
			}
		}
	}
	printk("SB IRQ Fail %d\n", sq );
	return FALSE;

sqsuss:

	dq = IRQ_GetBest( (bField|=(1<<sq)), nDefDIRQ );

	for(pin=1; pin<4; pin++)
	{
		SetPCISoundHardwareInt( dev, 1, 0xFF, 0xA+pin);

		for(i=0; i<4; i++)
		{
			if( PCIsetSetTestOn( i, dq ) )
			{
				SetPCISoundHardwareInt( dev, 1, dq, 0xA+pin);
				goto chipsuss;
			}
		}
	}
	printk("DREAM IRQ Fail %d\n", dq );
	return FALSE;

chipsuss :

	nDefSIRQ = sq;
	nDefDIRQ = dq;

	printk("SoundTrack 128DDMA : DREAM=0x%x IRQ=%d\n",
		nDefDRM, nDefDIRQ );

	printk("%18.18s", PCIsetGetChipString());
	printk(" : SB=0x%x WSS=0x%x IRQ=%d DMA=(%d,%d) OPL=0x%x MPU=0x%x\n",
		nDefSB, nDefWSS, nDefSIRQ, nDefPDMA, nDefCDMA, nDefOPL, nDefMPU );
	return TRUE;
}

